// RegUnRegTest.cpp
// Copyright (C) 2019 Katayama Hirofumi MZ <katayama.hirofumi.mz@gmail.com>
// This file is public domain software.
#include <windows.h>
#include <windowsx.h>
#include <stdio.h>

void ClassInfoTest(void)
{
    WNDCLASS wc;
    WNDCLASSEX wcx;
    BOOL ret;

    ret = GetClassInfo(NULL, TEXT("Ghost"), &wc);
    printf("GetClassInfo #1: 0x%04X\n", ret);
    ret = GetClassInfo(GetModuleHandle(NULL), TEXT("Ghost"), &wc);
    printf("GetClassInfo #2: 0x%04X\n", ret);

    ret = GetClassInfoEx(NULL, TEXT("Ghost"), &wcx);
    printf("GetClassInfoEx #1: 0x%04X\n", ret);
    ret = GetClassInfoEx(GetModuleHandle(NULL), TEXT("Ghost"), &wcx);
    printf("GetClassInfoEx #2: 0x%04X\n", ret);
}

LRESULT CALLBACK
WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

void RegTest(void)
{
    WNDCLASS wc;
    {
        ZeroMemory(&wc, sizeof(wc));
        wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
        wc.lpfnWndProc = WindowProc;
        wc.hInstance = NULL;    //
        wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
        wc.lpszClassName = TEXT("Ghost");
        ATOM atom = RegisterClass(&wc);
        DWORD dwError = GetLastError();
        printf("RegisterClass #1: 0x%04X\n", atom);
        printf("dwError #1: %ld\n", dwError);
    }
    {
        ZeroMemory(&wc, sizeof(wc));
        wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
        wc.lpfnWndProc = WindowProc;
        wc.hInstance = GetModuleHandle(NULL);   //
        wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
        wc.lpszClassName = TEXT("Ghost");
        ATOM atom = RegisterClass(&wc);
        DWORD dwError = GetLastError();
        printf("RegisterClass #2: 0x%04X\n", atom);
        printf("dwError #2: %ld\n", dwError);
    }
    {
        ZeroMemory(&wc, sizeof(wc));
        wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
        wc.lpfnWndProc = WindowProc;
        wc.hInstance = NULL;    //
        wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
        wc.lpszClassName = TEXT("Ghost");
        ATOM atom = RegisterClass(&wc);
        DWORD dwError = GetLastError();
        printf("RegisterClass #3: 0x%04X\n", atom);
        printf("dwError #3: %ld\n", dwError);
    }
    {
        ZeroMemory(&wc, sizeof(wc));
        wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
        wc.lpfnWndProc = WindowProc;
        wc.hInstance = GetModuleHandle(NULL);   //
        wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wc.hCursor = LoadCursor(NULL, IDC_ARROW);
        wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1);
        wc.lpszClassName = TEXT("Ghost");
        ATOM atom = RegisterClass(&wc);
        DWORD dwError = GetLastError();
        printf("RegisterClass #4: 0x%04X\n", atom);
        printf("dwError #4: %ld\n", dwError);
    }
}

void UnRegTest(void)
{
    BOOL ret;

    ret = UnregisterClass(TEXT("Ghost"), NULL);
    printf("UnregisterClass #1: %d\n", ret);

    ret = UnregisterClass(TEXT("Ghost"), GetModuleHandle(NULL));
    printf("UnregisterClass #2: %d\n", ret);

    ret = UnregisterClass(TEXT("Ghost"), NULL);
    printf("UnregisterClass #3: %d\n", ret);

    ret = UnregisterClass(TEXT("Ghost"), GetModuleHandle(NULL));
    printf("UnregisterClass #4: %d\n", ret);
}

int main(void)
{
    ClassInfoTest();
    RegTest();
    UnRegTest();
    return 0;
}
